define([
    "jquery",
    "./button.state"
],
function(
    jq,
    ButtonState
){

    /**
     * @constructor
     */
    return function( options ){

        var buttonColors = {};
        buttonColors[ ButtonState.Default ] = options.colors.default;
        buttonColors[ ButtonState.Hover ] = options.colors.hover;
        buttonColors[ ButtonState.Pressed ] = options.colors.pressed;
        buttonColors[ ButtonState.Inactive ] = options.colors.inactive;

        function createSvgPath() {
            var path = '<svg id="' + options.id + '" width="' + options.width + 'px" height="' + options.height + 'px" viewBox="0 0 50 50" preserveAspectRatio="none" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">' +
                '<title>' + options.caption + '</title>' +
                '<g class="NavigationButton" fill="' + options.colors.default.bg + '">' +
                '<rect x="0" y="0" width="50" height="50">' +
                '</g><g>';

            switch ( options.mode ) {
                case "PLAY":
                    path = path + '<polygon class="NavigationButtonIcon" fill="' + options.colors.default.icon + '" points="33 25 19 35 19 15 33 25">';
                    break;
                case "STOP":
                    path = path + '<rect x="17" y="17" width="16" height="16" fill="' + options.colors.default.icon + '"/>';
                    break;
                case "PAUSE":
                    path = path + '<rect x="19" y="14" width="4" height="22" fill="' + options.colors.default.icon + '"/>';
                    path = path + '<rect x="27" y="14" width="4" height="22" fill="' + options.colors.default.icon + '"/>';
                    break;
                default:
                    break;
            }

            path = path +
                '</g>' +
                '</svg>';

            /*'<svg id="' + options.id + '" width="' + options.width + 'px" height="' + (options.height * 4) + 'px" viewBox="0 0 44 176" preserveAspectRatio="none" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" style="background-color: transparent;">' +
                '<title>' + options.caption + '</title>' +
                '<defs></defs>' +
                '<g stroke="none" stroke-width="1" fill="none" fill-rule="evenodd">' +
                '<g>' +
                '<g class="NavigationButton" transform="translate(0.000000, 0.000000)" fill="' + options.colors.default.bg + '">' +
                '<rect x="0" y="0" width="44" height="44"></rect>' +
                '<polygon class="NavigationButtonIcon" fill="' + options.colors.default.icon + '" fill-rule="nonzero" transform="translate(22.500000, 22.000000) rotate(-360.000000) translate(-22.500000, -22.000000) " points="15 12 30 21.9999994 15 32"></polygon>' +
                '</g>' +
                '</g>' +
                '</g>' +
                '</svg>';*/

            return path;
        }

        function createButton(){

            var currentState = ButtonState.Default,
                svgButton = jq(document.createElement("div")),
                svg = svgButton.html( createSvgPath() )
                    .contents()
                    .addClass("sc-navigation")
                    .css({
                        "pointer-events" : "none"
                    })
                    .on("touchmove", function(e) {
                        e.preventDefault ? e.preventDefault() : true;
                    })
                    .get(0);

            var svgWrapper = jq(document.createElement("div"))
                .attr({
                    "title" : options.caption,
                    "class" : currentState
                })
                .css({
                    "cursor": "pointer",
                    "width": options.width + "px",
                    "height": options.height + "px",
                    "overflow": "hidden"
                })
                .append(svg);

            return {
                getHotspotElement : function(){
                    return svgWrapper.get(0);
                },
                hasState : function( buttonState ){
                    return buttonState === currentState;
                },
                changeState : function( buttonState ){

                    if( currentState === buttonState ){
                        return;
                    }

                    var colors = buttonColors[buttonState],
                        jqSvg = jq(svg);

                    svgWrapper.removeClass( currentState );
                    currentState = buttonState;
                    svgWrapper.addClass( buttonState );

                    jqSvg.find("[class=NavigationButton]").attr( "fill", colors.bg );
                    jqSvg.find("[class=NavigationButtonIcon]").attr( "fill", colors.icon );
                    jqSvg.css({
                        "cursor" : buttonState !== ButtonState.Inactive ? "pointer" : "default"
                    });
                }
            };
        }

        return {
            build : createButton
        };
    };
});
